/*
 * Copyright (c) 2010 Paul Jungwirth.
 * Licensed under the Apache License, Version 2.0:
 * http://www.apache.org/licenses/LICENSE-2.0.html
 */

package com.illuminated.encc;

import java.io.*;
import java.util.*;

public class AddSubExtensionNaming implements OutputNamingStrategy {

	public Map<File, File> getOutputFiles(File[] files, Object... params) throws OutputNamingException {
		Map<File,File> ret = new HashMap<File,File>();

		if (params == null || params.length == 0 || params[0] == null) {
			throw new OutputNamingException("no-extension");
		} else {
			String ext = (String)params[0];
			File outDir = (File)params[1];
			ext = ext.trim();
			if ("".equals(ext)) {
				throw new OutputNamingException("no-extension");
			} else {
				List<File>	badFiles = new ArrayList<File>();
				for (File f : files) {
					String inputName = f.getName();
					int pos = inputName.lastIndexOf(".");
					if (pos == -1) {
						badFiles.add(f);
					} else {
						String outputName = inputName.substring(0, pos) + ext + inputName.substring(pos);
						ret.put(f, new File(outDir, outputName));
					}
				}
				if (badFiles.size() > 0) {
					StringBuilder buf = new StringBuilder();
					for (File f : badFiles) {
						buf.append(f.getName());
						buf.append("\n");
					}
					throw new OutputNamingException("missing-extension", buf.toString());
				} else {
					return ret;
				}
			}
		}
	}

}
