/*
 * Copyright (c) 2010 Paul Jungwirth.
 * Licensed under the Apache License, Version 2.0:
 * http://www.apache.org/licenses/LICENSE-2.0.html
 */


package com.illuminated.encc;

import java.io.*;
import java.util.*;
import java.nio.charset.*;

public class Clui {

	private EncFilter _f = new EncFilter();
	public void setEncFilter(EncFilter f) { _f = f; }
	private String _encIn = "Cp1252";
	private String _encOut = "UTF-8";
	private InputStream _in = null;
	private OutputStream _out = null;
	private int _buffer = 0;
	private EncMapper _encMapper = new EncMapper();

	public static void main(String[] args) {
		Clui me = new Clui();
		System.exit(me.run(args));
	}

	public int run(String[] args) {
		parseArgs(args);
		try {
			_f.run(_encIn, _encOut, _in, _out, _buffer);
			return 0;
		} catch (CrippledException e) {
			System.err.println("Please register encc to use the " + e.getEncoding() + " encoding.");
			return 1;
		} catch (IOException e) {
			e.printStackTrace();
			return 1;
		}
	}

	public void carpBuffer() {
		System.err.println("Please use a number (e.g. 1024) for -b");
		System.err.println();
		usage();
	}

	public void parseArgs(String[] args) {
		int len = args.length;
		String a;
		boolean inOptions = true;
		for (int i = 0; i < len; i++) {
			a = args[i];
			if (inOptions) {
				if (a.equals("--enc-in")) {
					if (i + 1 < len) _encIn = _encMapper.get(args[++i]);
					else usage();

				} else if (a.equals("--enc-out")) {
					if (i + 1 < len) _encOut = _encMapper.get(args[++i]);
					else usage();

				} else if (a.equals("--help")) {
					usage(System.out, 0);

				} else if (a.equals("--buffer")) {
					try {
						if (i + 1 < len) _buffer = Integer.parseInt(args[++i]);
						else usage();
					} catch (NumberFormatException e) {
						carpBuffer();
					}
				} else if (a.startsWith("--buffer=")) {
					try {
						_buffer = Integer.parseInt(a.substring("--buffer=".length()));
					} catch (NumberFormatException e) {
						carpBuffer();
					}
				} else if (a.startsWith("--buffer")) {
					carpBuffer();

				} else if (a.equals("--")) {
					inOptions = false;

				} else if (a.equals("-")) {
					_in = System.in;
					inOptions = false;

				} else if (a.startsWith("-")) {
					int strlen = a.length();
					char[] str = a.toCharArray();
					char letter;
					for (int j = 1; j < strlen; j++) {
						letter = str[j];
						switch (letter) {
							case 'i':
								if (j + 1 == strlen && i + 1 < len) _encIn = _encMapper.get(args[++i]);
								else usage();
								break;
							case 'o':
								if (j + 1 == strlen && i + 1 < len) _encOut = _encMapper.get(args[++i]);
								else usage();
								break;
							case 'h':
								usage(System.out, 0);
								break;
							case 'e':
								printEncodings();
								break;
							case 'E':
								printMajorEncodings();
								break;
							case 'b':
								try {
									if (j + 1 == strlen && i + 1 < len) _buffer = Integer.parseInt(args[++i]);
									else usage();
								} catch (NumberFormatException e) {
									carpBuffer();
								}
								break;
							default:
								usage();
						}
					}
				} else {
					inOptions = false;
					try {
						_in = new FileInputStream(a);
					} catch (IOException e) {
						e.printStackTrace();
						System.exit(1);
					}
				}

			// Done with options:
			} else {

				if (_in == null) {
					if (a.equals("-")) {
						_in = System.in;
					} else {
						try {
							_in = new FileInputStream(a);
						} catch (IOException e) {
							e.printStackTrace();
							System.exit(1);
						}
					}
				} else if (_out == null) {
					if (a.equals("-")) {
						_out = System.out;
					} else {
						try {
							_out = new FileOutputStream(a);
						} catch (IOException e) {
							e.printStackTrace();
							System.exit(1);
						}
					}
				} else {
					usage();
				}
			}
		}
		
		if (_in == null) _in = System.in;
		if (_out == null) _out = System.out;
	}

	public void tryCharset(String charset) {
		try {
			Charset.forName(charset);
		} catch (IllegalCharsetNameException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (UnsupportedCharsetException e) {
			e.printStackTrace();
			System.exit(1);
		}
	}

	public void usage() {
		usage(System.err, 1);
	}

	public void usage(PrintStream out) {
		out.println("USAGE: encc [options] [in] [out]");
		out.println("options:");
		out.println("  -i --enc-in              The input encoding (default: Windows)");
		out.println("  -o --enc-o               The output encoding (default: UTF-8)");
		out.println("  -h --help                Print this message");
		out.println("  -e                       Print this message and all the supported encodings");
		out.println("  -E                       Print this message and the major supported encodings");
		out.println("  -b size --buffer=size    The buffer size");
		out.println("  --                       End options");
		out.println("in:    The file to read or - for stdin");
		out.println("out:   The file to write or - for stdout");
	}

	public void usage(PrintStream out, int exit) {
		usage(out);
		System.exit(exit);
	}

	public void printEncodings() {
		SortedMap m;
		String charset;
		Iterator i;
		Charset ch;

		// usage(System.out);
		// System.out.println();
		System.out.println("All supported encodings:");
		System.out.println();
		m = Charset.availableCharsets();
		i = m.keySet().iterator();
		while (i.hasNext()) {
			charset = (String)i.next();
			ch = (Charset)m.get(charset);
			System.out.println(charset + (ch.canEncode() ? " " : " *"));
		}
		System.out.println("* = decoding only");
		System.exit(0);
	}

	public void printMajorEncodings() {
		SortedMap m;

		// usage(System.out);
		// System.out.println();
		System.out.println("Major supported encodings:");
		System.out.println("(Equivalent names are given on the same line separated by spaces.)");
		System.out.println("(No name is case-sensitive.)");
		System.out.println();
		m = Charset.availableCharsets();
		if (m.containsKey("US-ASCII")) {
			System.out.println("US-ASCII ASCII");
		}
		if (m.containsKey("windows-1250")) {
			System.out.println("Windows-1250 Windows");
		}
		if (m.containsKey("x-MacRoman")) {
			System.out.println("x-MacRoman MacRoman Mac");
		}
		if (m.containsKey("ISO-8859-1")) {
			System.out.println("ISO-8859-1");
		}
		if (m.containsKey("UTF-8")) {
			System.out.println("UTF-8 UTF8");
		}
		if (m.containsKey("UTF-16BE")) {
			System.out.println("UTF-16BE UTF16BE UTF-16-BigEndian");
		}
		if (m.containsKey("UTF-16LE")) {
			System.out.println("UTF-16LE UTF16LE UTF-16-LittleEndian");
		}
		if (m.containsKey("UTF-16")) {
			System.out.println("Unicode UTF-16 UTF16 UTF-16-ByteOrderMark");
		}
		System.exit(0);
	}

}
