/*
 * Copyright (c) 2010 Paul Jungwirth.
 * Licensed under the Apache License, Version 2.0:
 * http://www.apache.org/licenses/LICENSE-2.0.html
 */

package com.illuminated.encc;

import javax.swing.*;
import javax.swing.event.*;
import com.illuminated.common.swing.*;
import com.illuminated.common.*;
import java.nio.charset.*;
import java.util.*;
import java.util.regex.*;
import java.text.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

// TODO: cripple UTF-8 encoding
// TODO: package it up and test on windows
// TODO: let people save their regexes.
// TODO: use launch4j
//       - get the gui to work. It's not finding the class. Test it on XP; looks like the same non-launch4j problem.
//       - little windows icon (e.g. in taskbar and folders with list view).
//       - medium windows icon (e.g. on desktop).
//       - title, etc. in process listing

public class Gui extends JFrame {
	private static final long serialVersionUID = 42L;

	private EncFilter _f = new EncFilter();
	public void setEncFilter(EncFilter f) { _f = f; }

	private ResourceBundle _bundle = BundleUtil.getBundle(Gui.class);
	private Action _okAction, _cancelAction, _chooseInputsAction, _clearAction, _changeExtensionAction,
			_addExtensionAction, _addSubExtensionAction, _newFileAction, _useRegexAction, _inPlaceAction,
			_testRegexAction;
	private JButton _ok, _cancel, _chooseInputs, _clear, _testRegex, _outputDir, _sameAsInputDir;
	private JRadioButton _changeExtension, _addExtension, _addSubExtension, _newFile, _useRegex, _inPlace;
	private JFileChooser _fileChooser, _outputDirChooser;
	private FilesList _inputFiles;
	private JTextField _changeExtensionField, _addExtensionField, _addSubExtensionField, _newFileField, _useRegexInField, _useRegexOutField, _outputDirField;
	private EasyComboBox _chooseEncIn, _chooseEncOut;
	private Map<String,String> _addSubExtensions, _addExtensions, _changeExtensions;
	private EasyFocusTraversalPolicy _traversal = new EasyFocusTraversalPolicy();
	private boolean _manuallySetOutputDirectory = false;
	private EncMapper _encMapper = new EncMapper();
	private JCheckBox _appendIfAbsent;
	private AppDefaults _defaults;
	private boolean _dontWarnAboutInPlace;

	public static void main(String[] args) {
		System.setProperty("swing.aatext", "true");
		Gui me = new Gui();
		me.build();
		me.pack();
		me.setVisible(true);
	}

	public Gui() {
	}

	/**
	 * File picker for input files. Show all the files selected so far.
	 * Radio buttons for output files:
	 *   - add extension
	 *   - add sub-extension
	 *   - change extension
	 *   - new file
	 *   - regex
	 *   - replace
	 *
	 *   default is "add sub-extension": win, mac, uni, utf8, enc
	 *
	 *   refuse if some input files don't match the regex.
	 *	 warn if some files would be replaced.
	 *	 warn if user selects the "replace" radio button and there isn't enough space in temp.
	 */
	public void build() {

// ugly:
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.BrownSugar());
// ugly:
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.DarkStar());
// like normal, but some color:
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.DesertBlue());
// eh, a little too blue:
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.DesertBluer());
// ugly:
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.DesertGreen());
// ugly
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.DesertRed());
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.DesertYellow());
// nice and colorful:
com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.ExperienceBlue());
// also nice:
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.ExperienceGreen());
// too gray
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.Silver());
// still pretty gray
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.SkyBlue());
// ugly:
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.SkyBluer());
// I kind of like this one:
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.SkyBluerTahoma());
// blech:
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.SkyGreen());
// ugly:
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.SkyKrupp());
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.SkyPink());
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.SkyRed());
// com.jgoodies.plaf.plastic.PlasticLookAndFeel.setMyCurrentTheme(new com.jgoodies.plaf.plastic.theme.SkyYellow());
		try {
			// UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			// UIManager.setLookAndFeel("com.jgoodies.plaf.windows.ExtWindowsLookAndFeel");
			// UIManager.setLookAndFeel("com.jgoodies.plaf.plastic.PlasticXPLookAndFeel");
			// UIManager.setLookAndFeel("com.jgoodies.plaf.plastic.Plastic3DLookAndFeel");
			UIManager.setLookAndFeel("com.jgoodies.plaf.plastic.PlasticLookAndFeel");
		} catch (Exception e) {
			// e.printStackTrace();
			// oh well
		}
		java.net.URL url = Gui.class.getClassLoader().getResource("eta.png");
		if (url != null) {
			setIconImage(new ImageIcon(url).getImage());
		} else {
			// System.out.println("didn't find icon");
		}
		Box	box;
		JLabel label;
		JPanel encPanel, encInPanel, encOutPanel, inputFilesPanel, inputFilesButtonPanel, outputFilesPanel, filesPanel;
		JScrollPane inputFilesScroll;
		String charset;
		Iterator	i;
		Map m;
		ButtonGroup	outputButtonGroup;
		_defaults = new AppDefaults(Gui.class.getName() + "_bundle");

		_okAction = new Command(this, "ok", _defaults);
		_cancelAction = new Command(this, "cancel", _defaults);
		_chooseInputsAction = new Command(this, "chooseInputs", _defaults);
		_clearAction = new Command(this, "clear", _defaults);

		_addSubExtensionAction = new Command(this, "chooseRadio", _defaults, "addSubExtension");
		_addExtensionAction = new Command(this, "chooseRadio", _defaults, "addExtension");
		_changeExtensionAction = new Command(this, "chooseRadio", _defaults, "changeExtension");
		_newFileAction = new Command(this, "chooseRadio", _defaults, "newFile");
		_useRegexAction = new Command(this, "chooseRadio", _defaults, "useRegex");
		_inPlaceAction = new Command(this, "chooseRadio", _defaults, "inPlace");

		_testRegexAction = new Command(this, "testRegex", _defaults);

		_outputDirField = new JTextField();
		_outputDirField.setColumns(30);
		_outputDirField.setEnabled(false);
		_sameAsInputDir = new JButton(new Command(this, "sameAsInputDir", _defaults));

		_addSubExtensionField = new JTextField();
		_addSubExtensionField.setColumns(10);
		_addSubExtensionField.setEnabled(true);
		_addSubExtensionField.setText(".utf8");
		_addSubExtensionField.addFocusListener(new Command(this, "rememberExtension", _defaults, null));
		_addExtensionField = new JTextField();
		_addExtensionField.setColumns(10);
		_addExtensionField.setEnabled(false);
		_addExtensionField.setText(".utf8");
		_addExtensionField.addFocusListener(new Command(this, "rememberExtension", _defaults, null));
		_changeExtensionField = new JTextField();
		_changeExtensionField.setColumns(10);
		_changeExtensionField.setEnabled(false);
		_changeExtensionField.setText(".utf8");
		_changeExtensionField.addFocusListener(new Command(this, "rememberExtension", _defaults, null));
		_newFileField = new JTextField();
		_newFileField.setColumns(10);
		_newFileField.setEnabled(false);
		_useRegexInField = new JTextField();
		_useRegexInField.setColumns(10);
		_useRegexInField.setEnabled(false);
		_useRegexOutField = new JTextField();
		_useRegexOutField.setColumns(10);
		_useRegexOutField.setEnabled(false);
		
		setTitle(_bundle.getString("title"));

		m = Charset.availableCharsets();

		setLayout(new BorderLayout());

		// Choose input encoding

		encInPanel = new JPanel();
		encInPanel.setLayout(new GridLayout(0, 1));
		label = new JLabel(_bundle.getString("encIn"));
		_chooseEncIn = new EasyComboBox();
		_traversal.add(_chooseEncIn);
		i = _encMapper.getDisplay().iterator();
		while (i.hasNext()) {
			charset = (String)i.next();
			_chooseEncIn.addItem(charset);
		}
		i = m.keySet().iterator();
		while (i.hasNext()) {
			charset = (String)i.next();
			_chooseEncIn.addItem(charset);
		}
		_chooseEncIn.addActionListener(new Command(this, "setEncIn", _defaults));
		encInPanel.add(label);
		encInPanel.add(_chooseEncIn);

		// Choose output encoding

		encOutPanel = new JPanel();
		encOutPanel.setLayout(new GridLayout(0, 1));
		label = new JLabel(_bundle.getString("encOut"));
		_chooseEncOut = new EasyComboBox();
		_traversal.add(_chooseEncOut);
		i = _encMapper.getDisplay().iterator();
		while (i.hasNext()) {
			charset = (String)i.next();
			_chooseEncOut.addItem(charset);
		}
		i = m.keySet().iterator();
		while (i.hasNext()) {
			charset = (String)i.next();
			if (((Charset)m.get(charset)).canEncode()) _chooseEncOut.addItem(charset);
		}
		_chooseEncOut.addActionListener(new Command(this, "setEncOut", _defaults));
		encOutPanel.add(label);
		encOutPanel.add(_chooseEncOut);

		encPanel = new JPanel();
		encPanel.add(encInPanel);
		encPanel.add(encOutPanel);
		add(encPanel, BorderLayout.NORTH);

		// Choose input files

		_fileChooser = new JFileChooser();
		_fileChooser.addChoosableFileFilter(new FileNameExtensionFilter(_bundle.getString("filter.txt.name"), "txt"));
		_fileChooser.setFileFilter(_fileChooser.getAcceptAllFileFilter());
		_fileChooser.setMultiSelectionEnabled(true);

		inputFilesPanel = new JPanel(new BorderLayout());
		inputFilesPanel.setBorder(BorderFactory.createTitledBorder(_bundle.getString("inputFiles")));

		_chooseInputs = new JButton(_chooseInputsAction);
		_clear = new JButton(_clearAction);
		_traversal.add(_chooseInputs);
		_traversal.add(_clear);
		inputFilesButtonPanel = PanelUtil.leftAlignedFlow(new Component[] {_chooseInputs, _clear});

		_inputFiles = new FilesList();
		_inputFiles.setRows(10);
		inputFilesScroll = new JScrollPane(_inputFiles);

		inputFilesPanel.add(inputFilesButtonPanel, BorderLayout.NORTH);
		inputFilesPanel.add(inputFilesScroll, BorderLayout.CENTER);

		// Choose output files

		_outputDirChooser = new JFileChooser();
		_outputDirChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

		GridBagLayout gridBag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		outputFilesPanel = new JPanel(gridBag);
		outputFilesPanel.setBorder(BorderFactory.createTitledBorder(_bundle.getString("outputFiles")));

		_outputDir = new JButton(new Command(this, "setOutputDir", _defaults));
		_addSubExtension = new JRadioButton(_addSubExtensionAction);
		_addExtension = new JRadioButton(_addExtensionAction);
		_changeExtension = new JRadioButton(_changeExtensionAction);
		_newFile = new JRadioButton(_newFileAction);
		_useRegex = new JRadioButton(_useRegexAction);
		_inPlace = new JRadioButton(_inPlaceAction);

		_addSubExtension.setSelected(true);
		outputButtonGroup = new ButtonGroup();
		outputButtonGroup.add(_addSubExtension);
		outputButtonGroup.add(_addExtension);
		outputButtonGroup.add(_changeExtension);
		outputButtonGroup.add(_newFile);
		outputButtonGroup.add(_useRegex);
		outputButtonGroup.add(_inPlace);

		c.weightx = 1;
		// c.weighty = 1;
		c.insets = new Insets(0, 5, 0, 5);
		c.fill = GridBagConstraints.NONE;

		label = new JLabel(_bundle.getString("outputDirectory"));
		c.gridwidth = 2;
		c.fill = GridBagConstraints.HORIZONTAL;
		gridBag.setConstraints(label, c);
		outputFilesPanel.add(label);

		c.gridwidth = 2;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.EAST;
		gridBag.setConstraints(_outputDir, c);
		outputFilesPanel.add(_outputDir);

		c.weightx = 0;
		c.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(_sameAsInputDir, c);
		outputFilesPanel.add(_sameAsInputDir);
		c.weightx = 1;

		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.CENTER;
		c.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(_outputDirField, c);
		outputFilesPanel.add(_outputDirField);

		c.gridwidth = 2;
		gridBag.setConstraints(_inPlace, c);
		outputFilesPanel.add(_inPlace);

		label = new JLabel("");
		c.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(label, c);
		outputFilesPanel.add(label);

		c.gridwidth = 2;
		gridBag.setConstraints(_addSubExtension, c);
		outputFilesPanel.add(_addSubExtension);

		c.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(_addSubExtensionField, c);
		outputFilesPanel.add(_addSubExtensionField);

		c.gridwidth = 2;
		gridBag.setConstraints(_addExtension, c);
		outputFilesPanel.add(_addExtension);

		c.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(_addExtensionField, c);
		outputFilesPanel.add(_addExtensionField);

		c.gridwidth = 2;
		gridBag.setConstraints(_changeExtension, c);
		outputFilesPanel.add(_changeExtension);

		c.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(_changeExtensionField, c);
		outputFilesPanel.add(_changeExtensionField);

		c.gridwidth = 2;
		label = new JLabel("    ");
		gridBag.setConstraints(label, c);
		outputFilesPanel.add(label);

		_appendIfAbsent = new JCheckBox(new Command(null, "append-if-absent", _defaults));
		_appendIfAbsent.setSelected(true);
		_appendIfAbsent.setEnabled(false);
		// c.gridwidth = 1;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 0;
		gridBag.setConstraints(_appendIfAbsent, c);
		outputFilesPanel.add(_appendIfAbsent);
		c.weightx = 1;

		/*
		label = new JLabel(_bundle.getString("append-if-absent"));
		c.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(label, c);
		outputFilesPanel.add(label);
		*/

		c.gridwidth = 2;
		gridBag.setConstraints(_newFile, c);
		outputFilesPanel.add(_newFile);

		c.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(_newFileField, c);
		outputFilesPanel.add(_newFileField);

		c.gridwidth = 2;
		gridBag.setConstraints(_useRegex, c);
		outputFilesPanel.add(_useRegex);

		label = new JLabel(_bundle.getString("regex-in"));
		c.gridwidth = 1;
		c.weightx = 0;
		gridBag.setConstraints(label, c);
		outputFilesPanel.add(label);
		c.weightx = 1;

		c.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(_useRegexInField, c);
		outputFilesPanel.add(_useRegexInField);

		_testRegex = new JButton(_testRegexAction);
		_testRegex.setEnabled(false);
		c.gridwidth = 2;
		c.fill = GridBagConstraints.NONE;
		gridBag.setConstraints(_testRegex, c);
		outputFilesPanel.add(_testRegex);
		c.fill = GridBagConstraints.HORIZONTAL;

		label = new JLabel(_bundle.getString("regex-out"));
		c.gridwidth = 1;
		c.weightx = 0;
		gridBag.setConstraints(label, c);
		outputFilesPanel.add(label);
		c.weightx = 1;

		c.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(_useRegexOutField, c);
		outputFilesPanel.add(_useRegexOutField);

		_traversal.add(_outputDir);
		_traversal.add(_sameAsInputDir);
		_traversal.add(_inPlace);
		_traversal.add(_addSubExtension);
		_traversal.add(_addSubExtensionField);
		_traversal.add(_addExtension);
		_traversal.add(_addExtensionField);
		_traversal.add(_changeExtension);
		_traversal.add(_changeExtensionField);
		_traversal.add(_appendIfAbsent);
		_traversal.add(_newFile);
		_traversal.add(_newFileField);
		_traversal.add(_useRegex);
		_traversal.add(_useRegexInField);
		_traversal.add(_useRegexOutField);
		_traversal.add(_testRegex);


		filesPanel = new JPanel(new GridLayout(0, 2));
		filesPanel.add(inputFilesPanel);
		filesPanel.add(outputFilesPanel);
		add(filesPanel, BorderLayout.CENTER);

		// Ok/Cancel buttons

		_cancel = new JButton(_cancelAction);
		_ok = new JButton(_okAction);
		_traversal.add(_cancel);
		_traversal.add(_ok);
		add(SwingUtil.okButtonPanel(_cancel, _ok), BorderLayout.SOUTH);

		configure();

		getRootPane().getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "cancel");
		getRootPane().getActionMap().put("cancel", _cancelAction);
		getRootPane().setDefaultButton(_ok);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
		setFocusTraversalPolicy(_traversal);
	}

	/**
	 * Sets up the initial look of the screen based on users' past choices.
	 */
	void configure() {
		Config c = new Config();
		c.load();

		switch(c.radioChoice) {
			case Config.RADIO_ADD_SUB:
				setupRadio(_addSubExtension);
				break;
			case Config.RADIO_ADD:
				setupRadio(_addExtension);
				break;
			case Config.RADIO_CHANGE:
				setupRadio(_changeExtension);
				break;
			case Config.RADIO_NEW_FILE:
				setupRadio(_newFile);
				break;
			case Config.RADIO_REGEX:
				setupRadio(_useRegex);
				break;
			case Config.RADIO_IN_PLACE:
				setupRadio(_inPlace);
				break;
			default:
				setupRadio(_addSubExtension);
		}

		_addSubExtensions = c.addSubExtension;
		_addExtensions = c.addExtension;
		_changeExtensions = c.changeExtension;

		_chooseEncIn.setSelectedItem(c.encIn);
		_chooseEncOut.setSelectedItem(c.encOut);
		setEncOut(c.encOut);

		_manuallySetOutputDirectory = c.manuallySetOutputDirectory;
		_appendIfAbsent.setSelected(c.appendIfAbsent);
		_dontWarnAboutInPlace = c.dontWarnAboutInPlace;

		if (c.inputDir != null) {
			File f = new File(c.inputDir);
			if (f.exists()) _fileChooser.setCurrentDirectory(f);
		}

		_outputDirChooser.setCurrentDirectory(_fileChooser.getCurrentDirectory());
		_outputDirField.setText(_bundle.getString("same-as-input"));
		if (_manuallySetOutputDirectory) {
			if (c.outputDir != null) {
				File f = new File(c.outputDir);
				if (f.exists()) {
					_outputDirChooser.setCurrentDirectory(f);
					_outputDirField.setText(f.getAbsolutePath());
				} else {
					_manuallySetOutputDirectory = false;
				}
			} else {
				_manuallySetOutputDirectory = false;
			}
		}
		_sameAsInputDir.setEnabled(_manuallySetOutputDirectory);

		if (c.newFile != null) {
			_newFile.setText(c.newFile);
		}

	}

	void saveConfiguration() {
		Config c = new Config();

		if (_addSubExtension.isSelected()) c.radioChoice = Config.RADIO_ADD_SUB;
		else if (_addExtension.isSelected()) c.radioChoice = Config.RADIO_ADD;
		else if (_changeExtension.isSelected()) c.radioChoice = Config.RADIO_CHANGE;
		else if (_newFile.isSelected()) c.radioChoice = Config.RADIO_NEW_FILE;
		else if (_useRegex.isSelected()) c.radioChoice = Config.RADIO_REGEX;
		else if (_inPlace.isSelected()) c.radioChoice = Config.RADIO_IN_PLACE;
		else c.radioChoice = Config.RADIO_ADD_SUB;

		c.appendIfAbsent = _appendIfAbsent.isSelected();
		c.newFile = _newFile.getText();
		c.regexIn = _useRegexInField.getText();
		c.regexOut = _useRegexOutField.getText();

		c.addSubExtension = _addSubExtensions;
		c.addExtension = _addExtensions;
		c.changeExtension = _changeExtensions;

		c.encIn = (String)_chooseEncIn.getSelectedItem();
		c.encOut = (String)_chooseEncOut.getSelectedItem();

		c.manuallySetOutputDirectory = _manuallySetOutputDirectory;
		c.inputDir = _fileChooser.getCurrentDirectory().getAbsolutePath();
		if (_manuallySetOutputDirectory) c.outputDir = _outputDirField.getText();
		else c.outputDir = null;

		c.dontWarnAboutInPlace = _dontWarnAboutInPlace;

		c.trySave();
	}

	void warn(String message) {
		JOptionPane.showMessageDialog(this, _bundle.getString(message + ".msg"),
				_bundle.getString(message + ".title"), JOptionPane.WARNING_MESSAGE);
	}

	void warn(String message, Object... o) {
		if (o == null || o.length == 0) {
			warn(message);
		} else {
			JOptionPane.showMessageDialog(this, MessageFormat.format(_bundle.getString(message + ".msg"), o),
					_bundle.getString(message + ".title"), JOptionPane.WARNING_MESSAGE);
		}
	}

	public Map<File,File> computeOutputFiles() {
		if (_inputFiles.countFiles() > 0) {
			try {
				// Factor out this method so we can test while avoiding popups:
				return nameOutputFiles(_inputFiles.getFiles());
			} catch (OutputNamingException e) {
				warn(e.getErrorKey(), e.getErrorParams());
			}
		} else {
			warn("no-input-files");
		}
		return new HashMap<File,File>();
	}

	public Map checkOutputFiles() {
		Map<File,File> ret = computeOutputFiles();

		if (_inPlace.isSelected()) {
			if (outputDirEqualsInputDir()) {
				if (_dontWarnAboutInPlace) {
					return ret;
				} else {
					CheckboxJOptionPane pane = CheckboxJOptionPane.confirm(_bundle.getString("in-place-warning"),
							new Command(null, "dont-warn-again", _defaults), JOptionPane.WARNING_MESSAGE);

					if (pane.use(this) != JOptionPane.OK_OPTION) {
						ret = new HashMap();
					}

					_dontWarnAboutInPlace = pane.isChecked();
					return ret;
				}
			} else {
				return ret;
			}
		} else {
			Set<File> inputs = ret.keySet();
			Set<File> outputs = new HashSet();
			Set<File> unchanged = new HashSet();
			Set<File> dups = new HashSet();
			for (File src : ret.keySet()) {
				File dest = ret.get(src);
				if (dest.equals(src)) {
					unchanged.add(dest);
				} else {
					if ( ! outputs.add(dest)) dups.add(dest);
					if (inputs.contains(dest)) dups.add(dest);
				}
			}
			if (unchanged.size() > 0 || dups.size() > 0) {
				StringBuilder buf = new StringBuilder();

				if (unchanged.size() > 0) {
					buf.append(_bundle.getString("unchanged-files"));
					for (File f : unchanged) {
						buf.append(f.getName());
						buf.append("\n");
					}
				}
				if (dups.size() > 0) {
					buf.append(_bundle.getString("duplicate-outputs"));
					for (File f : dups) {
						buf.append(f.getName());
						buf.append("\n");
					}
				}

				JOptionPane.showMessageDialog(this, buf.toString(),
						_bundle.getString("file-collisions"), JOptionPane.WARNING_MESSAGE);

				return new HashMap();
			} else {
				return ret;
			}

		}
	}

	boolean outputDirEqualsInputDir() {
		return _fileChooser.getCurrentDirectory().equals(getOutputDir());
	}

	public void ok(ActionEvent ev) {
		// TODO: warn about replacements that are on disk but not in the list of files to choose
		Map<File,File> m = checkOutputFiles();
		if (m.size() == 0) return;

		String encIn = _encMapper.get((String)_chooseEncIn.getSelectedItem());
		String encOut = _encMapper.get((String)_chooseEncOut.getSelectedItem());
		StringBuilder errors = new StringBuilder();

		for (Map.Entry<File,File> ent : m.entrySet()) {
			File in = ent.getKey();
			File out = ent.getValue();
			try {
				_f.run(encIn, encOut, new FileInputStream(in), new FileOutputStream(out), 0);
			} catch (CrippledException e) {
				errors.append(BundleUtil.format(_bundle, "crippled", e.getEncoding()));
			} catch (IOException e) {
				errors.append(in).append(" -> ").append(out).append(": ");
				errors.append(e.getMessage()).append("\n");
			}
		}
		if (_inPlace.isSelected() && outputDirEqualsInputDir()) {
			if (errors.length() == 0) {
				for (Map.Entry<File,File> ent : m.entrySet()) {
					ent.getValue().renameTo(ent.getKey());
				}
			}
		}

		if (errors.length() > 0) {
			warn("enc-errors", errors.toString());
		} else {
			JOptionPane.showMessageDialog(this, _bundle.getString("done.msg"),
					_bundle.getString("done.title"), JOptionPane.INFORMATION_MESSAGE);
		}
	}

	public void cancel(ActionEvent ev) {
		saveConfiguration();
		System.exit(0);
	}

	public void chooseInputs(ActionEvent ev) {
		int result;

		result = _fileChooser.showOpenDialog(this);
		if (result == JFileChooser.APPROVE_OPTION) {
			_inputFiles.setFiles(_fileChooser.getSelectedFiles());
			_testRegex.setEnabled(_useRegexInField.isEnabled() && _inputFiles.countFiles() > 0);
			if ( ! _manuallySetOutputDirectory) {
				_outputDirChooser.setCurrentDirectory(_fileChooser.getCurrentDirectory());
			}
		}
	}

	public void clear(ActionEvent ev) {
		_testRegex.setEnabled(false);
		_inputFiles.clearFiles();
	}

	public void chooseRadio(ActionEvent ev) {
		selectRadio((JRadioButton)ev.getSource());
	}

	public void selectRadio(JRadioButton b) {

		_addSubExtensionField.setEnabled(b == _addSubExtension);
		_addExtensionField.setEnabled(b == _addExtension);
		_changeExtensionField.setEnabled(b == _changeExtension);
		_appendIfAbsent.setEnabled(b == _changeExtension);
		_newFileField.setEnabled(b == _newFile);
		_testRegex.setEnabled(b == _useRegex && _inputFiles.countFiles() > 0);
		_useRegexInField.setEnabled(b == _useRegex);
		_useRegexOutField.setEnabled(b == _useRegex);

	}

	public void setupRadio(JRadioButton b) {
		b.setSelected(true);
		selectRadio(b);
	}

	public void testRegex(ActionEvent ev) {
		try {
			Map<File,File> m = new UseRegexNaming().getOutputFiles(_inputFiles.getFiles(),
					_useRegexInField.getText(), _useRegexOutField.getText(), getOutputDir());

			FileMappingDialog d = new FileMappingDialog(this);
			d.setFiles(m);
			d.build();
			d.pack();
			d.setVisible(true);
			if(d.shouldRun()) {
				ok(null);
			}
		} catch (OutputNamingException e) {
			warn(e.getErrorKey(), e.getErrorParams());
		}
	}

	public void setEncIn(ActionEvent ev) {
	}

	public void setEncOut(ActionEvent ev) {
		String s = (String)_chooseEncOut.getSelectedItem();
		setEncOut(s);
	}

	void setEncOut(String enc) {
		String ext;
		// System.out.println("setEncOut(" + enc + ")");
		// System.out.println(_addSubExtensions);
		
		ext = _addSubExtensions.get(enc);
		if (ext == null) ext = ".enc";
		_addSubExtensionField.setText(ext);

		// System.out.println(ext);

		ext = _addExtensions.get(enc);
		if (ext == null) ext = ".enc";
		_addExtensionField.setText(ext);

		ext = _changeExtensions.get(enc);
		if (ext == null) ext = ".enc";
		_changeExtensionField.setText(ext);
	}

	File getOutputDir() {
		if (_manuallySetOutputDirectory) {
			return new File(_outputDirField.getText());
		} else {
			return _inputFiles.getFiles()[0].getParentFile();
		}
	}

	Map<File,File> nameOutputFiles(File[] f) throws OutputNamingException {
		if (_addSubExtension.isSelected()) {
			return new AddSubExtensionNaming().getOutputFiles(f, _addSubExtensionField.getText(), getOutputDir());
		} else if (_addExtension.isSelected()) {
			return new AddExtensionNaming().getOutputFiles(f, _addExtensionField.getText(), getOutputDir());
		} else if (_changeExtension.isSelected()) {
			return new ChangeExtensionNaming().getOutputFiles(f, _changeExtensionField.getText(), getOutputDir(), _appendIfAbsent.isSelected());
		} else if (_newFile.isSelected()) {
			return new NewFileNaming().getOutputFiles(f, _newFileField.getText(), getOutputDir());
		} else if (_useRegex.isSelected()) {
			return new UseRegexNaming().getOutputFiles(f, _useRegexInField.getText(), _useRegexOutField.getText(),
					getOutputDir());
		} else if (_inPlace.isSelected()) {
			if (outputDirEqualsInputDir()) {
				return new InPlaceNaming().getOutputFiles(f);
			} else {
				return new InPlaceNaming().getOutputFiles(f, getOutputDir());
			}
		} else {
			throw new IllegalStateException("No naming strategy selected.");
		}
	}

	public void setOutputDir(ActionEvent ev) {
		int result;

		result = _outputDirChooser.showOpenDialog(this);
		if (result == JFileChooser.APPROVE_OPTION) {
			_manuallySetOutputDirectory = true;
			File f = _outputDirChooser.getSelectedFile();
			_outputDirField.setText(f == null ? "" : f.toString());
			_sameAsInputDir.setEnabled(true);
		}
	}

	public void rememberExtension(FocusEvent ev) {
		if (ev.getID() == FocusEvent.FOCUS_LOST) {
			Object f = ev.getSource();
			if (f == _addSubExtensionField) {
				_addSubExtensions.put((String)_chooseEncOut.getSelectedItem(), _addSubExtensionField.getText());
			} else if (f == _addExtensionField) {
				_addExtensions.put((String)_chooseEncOut.getSelectedItem(), _addExtensionField.getText());
			} else if (f == _changeExtensionField) {
				_changeExtensions.put((String)_chooseEncOut.getSelectedItem(), _changeExtensionField.getText());
			}
		}
	}

	public void sameAsInputDir(ActionEvent ev) {
		_manuallySetOutputDirectory = false;
		_outputDirChooser.setCurrentDirectory(_fileChooser.getCurrentDirectory());
		_outputDirField.setText(_bundle.getString("same-as-input"));
		_sameAsInputDir.setEnabled(false);
	}

}
