/*
 * Copyright (c) 2010 Paul Jungwirth.
 * Licensed under the Apache License, Version 2.0:
 * http://www.apache.org/licenses/LICENSE-2.0.html
 */

package com.illuminated.encc;

import java.io.*;
import java.util.*;

/**
 * Use this class when you don't want to change the names of the files.
 * It supports two scenarios:
 * <ol>
 *	 <li>You're altering the files in-place and need to write out to temp files</li>
 *	 <li>You're write the files to a different directory.</li>
 * </ol>
 * If you want the files in a new directory, pass a File argument representing
 * that directory as the first optional parameter to getOutputFiles.
 * <p>
 * If you want temp files, don't pass a directory parameter.
 */
public class InPlaceNaming implements OutputNamingStrategy {

	public Map<File, File> getOutputFiles(File[] files, Object... params) throws OutputNamingException {
		Map ret = new HashMap<File,File>();
		
		// We are writing the file in-place, so we must create temp files:
		if (params.length == 0 || params[0] == null) {
			for (File f : files) {
				try {
					File t = File.createTempFile("enc", "enc", f.getParentFile());
					t.deleteOnExit();
					ret.put(f, t);
				} catch (IOException e) {
					throw new OutputNamingException("cant-create-file", f, e.getMessage());
				}
			}

		// Write the files to a different directory:
		} else {
			File outputDir = (File)params[0];
			for (File f : files) {
				ret.put(f, new File(outputDir, f.getName()));
			}
		}

		return ret;
	}

}
