/*
 * Copyright (c) 2010 Paul Jungwirth.
 * Licensed under the Apache License, Version 2.0:
 * http://www.apache.org/licenses/LICENSE-2.0.html
 */

package com.illuminated.encc;

import java.io.*;
import java.util.*;
import java.util.regex.*;

public class UseRegexNaming implements OutputNamingStrategy {

	public Map<File, File> getOutputFiles(File[] files, Object... params) throws OutputNamingException {
		Map ret = new HashMap<File,File>();

		if (params == null || params.length < 2 || params[0] == null || params[1] == null) {
			throw new OutputNamingException("no-regex");
		} else {
			String pat = (String)params[0];
			pat = pat.trim();
			if ("".equals(pat)) {
				throw new OutputNamingException("no-regex");
			}
			String subst = (String)params[1];
			File outDir = (File)params[2];
			subst = subst.trim();
			if ("".equals(subst)) {
				throw new OutputNamingException("no-regex");
			}

			Pattern	p;
			try {
				p = Pattern.compile(pat);
			} catch (PatternSyntaxException e) {
				throw new OutputNamingException("bad-regex");
			}

			for (File f : files) {
				Matcher m = p.matcher(f.getName());
				String result = m.replaceFirst(subst);
				ret.put(f, new File(outDir, result));
			}
			return ret;
		}
	}

}
