package com.illuminated.encc;

import junit.framework.*;
import java.io.*;
import java.util.*;

public class AddExtensionNamingTest extends TestCase {


	public void testNoExtension() throws Exception {
		AddExtensionNaming naming = new AddExtensionNaming();
		File f = new File("a/f.txt");
		File dir = new File("a");

		try {
			naming.getOutputFiles(new File[] { f } , null, dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , "", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , new Object[0]);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , new Object[] { null, dir });
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , "   ", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , " \t\n  ", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}
	}

	public void testSameDirectory() throws Exception {
		AddExtensionNaming naming = new AddExtensionNaming();
		File[] f = new File[] {
			new File("a/f.txt"),
			new File("a/g.txt.old"),
			new File("a/h.1"),
			new File("a/i."),
			new File("a/j")
		};
		Map<File,File> m = naming.getOutputFiles(f, ".enc", new File("a"));
		assertEquals(5, m.size());
		assertEquals(new File("a/f.txt.enc"), m.get(f[0]));
		assertEquals(new File("a/g.txt.old.enc"), m.get(f[1]));
		assertEquals(new File("a/h.1.enc"), m.get(f[2]));
		assertEquals(new File("a/i..enc"), m.get(f[3]));
		assertEquals(new File("a/j.enc"), m.get(f[4]));
	}

	public void testDifferentDirectory() throws Exception {
		AddExtensionNaming naming = new AddExtensionNaming();
		File[] f = new File[] {
			new File("a/f.txt"),
			new File("a/g.txt.old"),
			new File("a/h.1"),
			new File("a/i."),
			new File("a/j")
		};
		Map<File,File> m = naming.getOutputFiles(f, ".enc", new File("b"));
		assertEquals(5, m.size());
		assertEquals(new File("b/f.txt.enc"), m.get(f[0]));
		assertEquals(new File("b/g.txt.old.enc"), m.get(f[1]));
		assertEquals(new File("b/h.1.enc"), m.get(f[2]));
		assertEquals(new File("b/i..enc"), m.get(f[3]));
		assertEquals(new File("b/j.enc"), m.get(f[4]));
	}

}
