package com.illuminated.encc;

import junit.framework.*;
import java.io.*;
import java.util.*;

public class AddSubExtensionNamingTest extends TestCase {


	public void testNoExtension() throws Exception {
		AddSubExtensionNaming naming = new AddSubExtensionNaming();
		File f = new File("a/f.txt");
		File dir = new File("a");

		try {
			naming.getOutputFiles(new File[] { f } , null, dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , "", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , new Object[0]);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , new Object[] { null, dir });
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , "   ", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , " \t\n  ", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}
	}

	public void testMissingExtension() throws Exception {
		AddSubExtensionNaming naming = new AddSubExtensionNaming();
		File[] f = new File[] {
			new File("a/f.txt"),
			new File("a/g"),
			new File("a/h"),
			new File("a/i.tex")
		};

		try {
			naming.getOutputFiles(f, ".enc", new File("a"));
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("missing-extension", e.getErrorKey());
			assertEquals("g\nh\n", e.getErrorParams()[0]);
		}
	}


	public void testSameDirectory() throws Exception {
		AddSubExtensionNaming naming = new AddSubExtensionNaming();
		File[] f = new File[] {
			new File("a/f.txt"),
			new File("a/g.txt.old"),
			new File("a/h.1"),
			new File("a/i.")
		};
		Map<File,File> m = naming.getOutputFiles(f, ".enc", new File("a"));
		assertEquals(4, m.size());
		assertEquals(new File("a/f.enc.txt"), m.get(f[0]));
		assertEquals(new File("a/g.txt.enc.old"), m.get(f[1]));
		assertEquals(new File("a/h.enc.1"), m.get(f[2]));
		assertEquals(new File("a/i.enc."), m.get(f[3]));
	}

	public void testDifferentDirectory() throws Exception {
		AddSubExtensionNaming naming = new AddSubExtensionNaming();
		File[] f = new File[] {
			new File("a/f.txt"),
			new File("a/g.txt.old"),
			new File("a/h.1"),
			new File("a/i.")
		};
		Map<File,File> m = naming.getOutputFiles(f, ".enc", new File("b"));
		assertEquals(4, m.size());
		assertEquals(new File("b/f.enc.txt"), m.get(f[0]));
		assertEquals(new File("b/g.txt.enc.old"), m.get(f[1]));
		assertEquals(new File("b/h.enc.1"), m.get(f[2]));
		assertEquals(new File("b/i.enc."), m.get(f[3]));
	}

}
