package com.illuminated.encc;

import junit.framework.*;
import java.io.*;
import java.util.*;

public class ChangeExtensionNamingTest extends TestCase {


	public void testNoExtension() throws Exception {
		ChangeExtensionNaming naming = new ChangeExtensionNaming();
		File f = new File("a/f.txt");
		File dir = new File("a");

		try {
			naming.getOutputFiles(new File[] { f } , null, dir, false);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , "", dir, false);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , new Object[0]);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , new Object[] { null, dir, false });
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , "   ", dir, false);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , " \t\n  ", dir, false);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-extension", e.getErrorKey());
		}
	}

	public void testMissingExtension() throws Exception {
		ChangeExtensionNaming naming = new ChangeExtensionNaming();
		File[] f = new File[] {
			new File("a/f.txt"),
			new File("a/g"),
			new File("a/h"),
			new File("a/i.tex")
		};

		try {
			naming.getOutputFiles(f, ".enc", new File("a"), false);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("missing-extension", e.getErrorKey());
			assertEquals("g\nh\n", e.getErrorParams()[0]);
		}

	}

	public void testMissingExtensionOkay() throws Exception {
		ChangeExtensionNaming naming = new ChangeExtensionNaming();
		Map<File,File> m;
		File[] f = new File[] {
			new File("a/f.txt"),
			new File("a/g"),
			new File("a/h"),
			new File("a/i.tex")
		};

		// same directory
		m = naming.getOutputFiles(f, ".enc", new File("a"), true);
		assertEquals(4, m.size());
		assertEquals(new File("a/f.enc"), m.get(f[0]));
		assertEquals(new File("a/g.enc"), m.get(f[1]));
		assertEquals(new File("a/h.enc"), m.get(f[2]));
		assertEquals(new File("a/i.enc"), m.get(f[3]));

		// different directory
		m = naming.getOutputFiles(f, ".enc", new File("b"), true);
		assertEquals(4, m.size());
		assertEquals(new File("b/f.enc"), m.get(f[0]));
		assertEquals(new File("b/g.enc"), m.get(f[1]));
		assertEquals(new File("b/h.enc"), m.get(f[2]));
		assertEquals(new File("b/i.enc"), m.get(f[3]));

	}



	public void testSameDirectory() throws Exception {
		ChangeExtensionNaming naming = new ChangeExtensionNaming();
		File[] f = new File[] {
			new File("a/f.txt"),
			new File("a/g.txt.old"),
			new File("a/h.1"),
			new File("a/i.")
		};
		Map<File,File> m = naming.getOutputFiles(f, ".enc", new File("a"), false);
		assertEquals(4, m.size());
		assertEquals(new File("a/f.enc"), m.get(f[0]));
		assertEquals(new File("a/g.txt.enc"), m.get(f[1]));
		assertEquals(new File("a/h.enc"), m.get(f[2]));
		assertEquals(new File("a/i.enc"), m.get(f[3]));
	}

	public void testDifferentDirectory() throws Exception {
		ChangeExtensionNaming naming = new ChangeExtensionNaming();
		File[] f = new File[] {
			new File("a/f.txt"),
			new File("a/g.txt.old"),
			new File("a/h.1"),
			new File("a/i.")
		};
		Map<File,File> m = naming.getOutputFiles(f, ".enc", new File("b"), false);
		assertEquals(4, m.size());
		assertEquals(new File("b/f.enc"), m.get(f[0]));
		assertEquals(new File("b/g.txt.enc"), m.get(f[1]));
		assertEquals(new File("b/h.enc"), m.get(f[2]));
		assertEquals(new File("b/i.enc"), m.get(f[3]));
	}

}
