package com.illuminated.encc;

import junitx.framework.ArrayAssert;
import junit.framework.*;
import java.io.*;

public class EncFilterTest extends TestCase {

	public static byte[] cast(int[] in) {
		int len = in.length;
		byte[] b = new byte[len];
		for (int i = 0; i < len; i++) {
			b[i] = (byte)(in[i] & 0xff);
		}
		return b;
	}

	public static byte[] toByte(String src) {
		src = src.replaceAll("\\s", "");
		int len = src.length() / 2;
		byte[] b = new byte[len];
		try {
			for (int i = 0; i < len; i++) {
				b[i] = (byte)Integer.parseInt(src.substring(2*i, 2*i + 2), 16);
			}
		} catch (NumberFormatException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
		return b;
	}

	// (c) (r) tm dagger double-dagger -- --- &lsquo; &rsquo; &ldquo; &rdquo;
	public static final byte[] funnyChars_unicode = toByte("00a9 00ae 2122 2020 2021 2013 2014 2018 2019 201c 201d");
	// public static final byte[] funnyChars_unicode = cast(new int[] {0x00, 0xa9, 0x00, 0xae, 0x21, 0x22, 0x20, 0x20, 0x20, 0x21, 0x20, 0x13, 0x20, 0x14, 0x20, 0x18, 0x20, 0x19, 0x20, 0x1c, 0x20, 0x1d});
	public static final byte[] funnyChars_utf8 = toByte("c2a9 c2ae e284a2 e280a0 e280a1 e28093 e28094 e28098 e28099 e2809c e2809d");
	public static final byte[] funnyChars_windows = toByte("a9 ae 99 86 87 96 97 91 92 93 94");
	public static final byte[] funnyChars_mac = toByte("a9 a8 aa a0 e0 d0 d1 d4 d5 d2 d3");

	public static final String john11 = "Ἐν ἀρχῇ ἦν ὁ λόγος.";
	public static final byte[] john11_utf8 = {};
	public static final byte[] john11_windows = {};
	public static final byte[] john11_mac = {};
	// TODO: give the bytes for each applicable encoding.
	// TODO: for each encoding, find some characters that are appropriate.

	public void testJohn11() throws Exception {
	}

	public void testFunnyChars() throws Exception {
		OutputStream out;
		File f;

		printBytes(funnyChars_unicode);

		// from unicode
		f = File.createTempFile("EncFilterTest", "tmp");
		f.deleteOnExit();
		out = new FileOutputStream(f);
		out.write(funnyChars_unicode, 0, funnyChars_unicode.length);
		out.close();

		convert(f, "UTF-16BE", "UTF-8", funnyChars_utf8);
		convert(f, "UTF-16BE", "windows-1250", funnyChars_windows);
		convert(f, "UTF-16BE", "x-MacRoman", funnyChars_mac);

		// from utf8
		f = File.createTempFile("EncFilterTest", "tmp");
		f.deleteOnExit();
		out = new FileOutputStream(f);
		out.write(funnyChars_utf8, 0, funnyChars_utf8.length);
		out.close();

		convert(f, "UTF-8", "UTF-16BE", funnyChars_unicode);
		convert(f, "UTF-8", "windows-1250", funnyChars_windows);
		convert(f, "UTF-8", "x-MacRoman", funnyChars_mac);

		// from windows
		f = File.createTempFile("EncFilterTest", "tmp");
		f.deleteOnExit();
		out = new FileOutputStream(f);
		out.write(funnyChars_windows, 0, funnyChars_windows.length);
		out.close();

		convert(f, "windows-1250", "UTF-16BE", funnyChars_unicode);
		convert(f, "windows-1250", "UTF-8", funnyChars_utf8);
		convert(f, "windows-1250", "x-MacRoman", funnyChars_mac);

		// from mac
		f = File.createTempFile("EncFilterTest", "tmp");
		f.deleteOnExit();
		out = new FileOutputStream(f);
		out.write(funnyChars_mac, 0, funnyChars_mac.length);
		out.close();

		convert(f, "x-MacRoman", "UTF-16BE", funnyChars_unicode);
		convert(f, "x-MacRoman", "UTF-8", funnyChars_utf8);
		convert(f, "x-MacRoman", "windows-1250", funnyChars_windows);
	}

	public void printBytes(byte[] b) {
		System.out.println(byteArrayToString(b));
	}

	public String byteArrayToString(byte[] b) {
		StringBuilder buf = new StringBuilder("{");
		for (int i = 0; i < b.length; i++) {
			if (i > 0) buf.append(", ");
			else buf.append(" ");
			buf.append(Integer.toHexString(b[i] & 0xff));
		}
		buf.append(" }");
		return buf.toString();
	}


	public void convert(File src, String encIn, String encOut, byte[] verify) throws Exception {
		File dest = File.createTempFile("EncFilterTest", "tmp");
		dest.deleteOnExit();
		InputStream in = new FileInputStream(src);
		OutputStream out = new FileOutputStream(dest);
		EncFilter enc = new EncFilter();

		enc.run(encIn, encOut, in, out, 0);

		in = new FileInputStream(dest);

		ArrayAssert.assertEquals(encIn + " TO " + encOut, verify, readAll(in, verify.length));
	}

	public void arrayCopy(byte[] src, int srcPos, byte[] dest, int destPos, int len) {
		for (int i = 0; i < len; i++) {
			dest[i+destPos] = src[i+srcPos];
		}
	}

	public void assertEquals(byte[] expected, byte[] actual) {
		boolean okay = true;
		if (expected.length != actual.length) {
			okay = false;
		} else {
			for (int i = 0; i < expected.length; i++) {
				if (expected[i] != actual[i]) {
					okay = false;
					break;
				}
			}
		}
		
		if (! okay) throw new AssertionFailedError("expected <" + byteArrayToString(expected) + "> got <" + byteArrayToString(actual) + ">");
		
	}

	public byte[] readAll(InputStream in, int size) throws Exception {
		byte[] buffer = new byte[size];
		byte[] b = new byte[size];
		int pos = 0;
		int len;

		while ((len = in.read(buffer, 0, size)) != -1) {
			arrayCopy(buffer, 0, b, pos, len);
			pos += len;
			// if we read too much, just throw an exception.
			printBytes(b);
		}
		in.close();

		return b;
	}

}
