package com.illuminated.encc;

import junit.framework.*;
import java.io.*;
import java.util.*;

public class OutputNamingTest extends TestCase {


	public void testNewFileNaming() throws Exception {
		NewFileNaming naming = new NewFileNaming();
		File f = new File("a/f");
		Map<File,File> m;
		File dir = new File("a");

		try {
			naming.getOutputFiles(new File[2], "newFile", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("too-many-input-files", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , null, dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-new-name", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , "", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-new-name", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , new Object[0]);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-new-name", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , new Object[] { null, dir });
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-new-name", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , "   ", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-new-name", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , " \t\n  ", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-new-name", e.getErrorKey());
		}

		m = naming.getOutputFiles(new File[] { f }, "newFile", dir);
		assertEquals(1, m.size());
		assertEquals(f, m.keySet().iterator().next());
		assertEquals(new File(f.getParentFile(), "newFile"), m.values().iterator().next());

		m = naming.getOutputFiles(new File[] { f }, "  newFile\t", dir);
		assertEquals(1, m.size());
		assertEquals(f, m.keySet().iterator().next());
		assertEquals(new File(f.getParentFile(), "newFile"), m.values().iterator().next());

		m = naming.getOutputFiles(new File[] { f }, "newFile", new File("b"));
		assertEquals(1, m.size());
		assertEquals(f, m.keySet().iterator().next());
		assertEquals(new File("b/newFile"), m.values().iterator().next());

		m = naming.getOutputFiles(new File[] { f }, "  newFile\t", new File("b"));
		assertEquals(1, m.size());
		assertEquals(f, m.keySet().iterator().next());
		assertEquals(new File("b/newFile"), m.values().iterator().next());

	}

}
