package com.illuminated.encc;

import junit.framework.*;
import java.io.*;
import java.util.*;

public class UseRegexNamingTest extends TestCase {


	public void testNoInPattern() throws Exception {
		UseRegexNaming naming = new UseRegexNaming();
		File f = new File("a/f.txt");
		File dir = new File("a");

		try {
			naming.getOutputFiles(new File[] { f } , null, "out", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-regex", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , "", "out", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-regex", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , new Object[0]);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-regex", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , new Object[] { null, "out", dir });
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-regex", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , "   ", "out", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-regex", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , " \t\n  ", "out", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-regex", e.getErrorKey());
		}
	}

	public void testNoOutPattern() throws Exception {
		UseRegexNaming naming = new UseRegexNaming();
		File f = new File("a/f.txt");
		File dir = new File("a");

		try {
			naming.getOutputFiles(new File[] { f } , "in", null, dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-regex", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , "in", "", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-regex", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , new Object[] { "in", null, dir });
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-regex", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , "in", "   ", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-regex", e.getErrorKey());
		}

		try {
			naming.getOutputFiles(new File[] { f } , "in", " \t\n  ", dir);
			fail("Expected exception");
		} catch (OutputNamingException e) {
			assertEquals("no-regex", e.getErrorKey());
		}
	}

	public void testSameDirectory() throws Exception {
		UseRegexNaming naming = new UseRegexNaming();
		File[] f = new File[] {
			new File("a/f.txt"),
			new File("a/g.txt.old"),
			new File("a/h.1"),
			new File("a/i."),
			new File("a/j")
		};
		Map<File,File> m = naming.getOutputFiles(f, "^(.*)\\.(.*)$", "$2.$1", new File("a"));
		assertEquals(5, m.size());
		assertEquals(new File("a/txt.f"), m.get(f[0]));
		assertEquals(new File("a/old.g.txt"), m.get(f[1]));
		assertEquals(new File("a/1.h"), m.get(f[2]));
		assertEquals(new File("a/.i"), m.get(f[3]));
		assertEquals(new File("a/j"), m.get(f[4]));
	}

	public void testDifferentDirectory() throws Exception {
		UseRegexNaming naming = new UseRegexNaming();
		File[] f = new File[] {
			new File("a/f.txt"),
			new File("a/g.txt.old"),
			new File("a/h.1"),
			new File("a/i."),
			new File("a/j")
		};
		Map<File,File> m = naming.getOutputFiles(f, "^(.*)\\.(.*)$", "$2.$1", new File("b"));
		assertEquals(5, m.size());
		assertEquals(new File("b/txt.f"), m.get(f[0]));
		assertEquals(new File("b/old.g.txt"), m.get(f[1]));
		assertEquals(new File("b/1.h"), m.get(f[2]));
		assertEquals(new File("b/.i"), m.get(f[3]));
		assertEquals(new File("b/j"), m.get(f[4]));
	}

}
